import influxdb, apsw, time

def dict_factory(cursor, row):
    d = dict()
    for idx,col in enumerate(cursor.description): d[col[0]] = row[idx]
    return d    

class InfluxContext(object):
    pass

class InfluxModule(object):

    def __init__(s, influxcontext):
        s.influxcontext = influxcontext

    def Create(s, apswcontext, modulename, database, measurement, *args):
        rset = s.influxcontext.session.query("select * from " + measurement + " limit 1")
        columns = list(rset.get_points())[0].keys()
        schema = 'create table %s (' + ','.join(columns) + ')'
        return schema % measurement, InfluxTable(measurement, apswcontext, s.influxcontext, columns)
    Connect = Create

class InfluxTable(object):

    def __init__(s, name, apswcontext, influxcontext, columns):
        s.influxcontext = influxcontext
        s.apswcontext = apswcontext
        s.columns = columns
        s.indexnum = -1
        s.index=[]
        

    def BestIndex(s, constraints, orderbys):
        constraints_used = range(len(constraints))
        s.indexnum = s.indexnum + 1
        s.index.append(constraints)
        return [constraints_used, s.indexnum, 'index', False, 1000]

    def Open(s):
        return InfluxCursor(s.apswcontext, s.influxcontext, s.columns, s.index)

    def Disconnect(s):
        pass

    def Destroy(s):
        raise Exception('Destroy(): Function not supported!')

    def UpdateChangeRow(s, rowid, newrowid, fields):
        raise Exception('UpdateChangeRow(): Function not supported!')

    def UpdateInsertRow(s, rowid, fields):
        raise Exception('UpdateInsertRow(): Function not supported!')

    def UpdateDeleteRow(s, rowid):
        raise Exception('UpdateDeleteRow(): Function not supported!')

class InfluxCursor(object):
    
    def __init__(s, apswcontext, influxcontext, columns, index):
        s.apswcontext = apswcontext
        s.influxcontext = influxcontext
        s.columns = columns
        s.index = index
    
    def Close(s):
        pass
    
    def Column(s, number):
        return s.next[s.columns[number]]

    def Eof(s):
        ret = False
        try: s.next = s.data.next()
        except: ret = True
        return ret
    
    def Filter(s, indexnum, indexname, constraintargs):
        constraints = s.index[indexnum]
        where = "" if len(constraints) == 0 else "where "
        i = -1
        for c in constraints:
            i += 1
            op = ' = ' if c[1] == 2 else ' <= ' if c[1] == 8 else ' < ' if c[1] == 16 else ' >= ' if c[1] == 32 else ' > '
            quote = "'" if type(constraintargs[i]) == type(u'A') else ""
            where += str(s.columns[c[0]]) + op + quote+ str(constraintargs[i]) + quote + ' and '
        where = where [:-5]
        request = 'select * from ' + s.influxcontext.measurement + ' ' + where
        s.data = s.influxcontext.session.query(request).get_points()
    
    def Next(s):
        pass
    
    def Rowid(s):
        raise Exception('Rowid(): Function not supported!')

class InfluxSQLClient(influxdb.InfluxDBClient):

    def __init__(s, *args, **kwargs):
        super(InfluxSQLClient,s).__init__(*args, **kwargs)
        s.apsw = apsw.Connection(':memory:')

    def switch_database(s, database):
        s.apsw.close()
        s.apsw = apsw.Connection(':memory:')
        ret = super(InfluxSQLClient,s).switch_database(database)
        s.database = database
        return ret

    def detach(s, table):
        s.apsw.cursor().execute("drop table if exists " + table)

    def attach(s, table):
        if table not in [x['name'] for x in s.query("show measurements").get_points()]: raise Exception('Unknown measurement: ' + table)
        context = InfluxContext()
        context.session = s
        context.database = s.database
        context.measurement = table
        s.apsw.createmodule('influxdb_' + table, InfluxModule(context))
        s.apsw.cursor().execute("create virtual table " + table + " using influxdb_" + table + "()")

    def sql(s, query):
        return s.apsw.cursor().execute(query)
    
    def imp(s, category='sqlite', file=None, table=None, extracol='value', batch=5000):
        if category.lower() not in ['sqlite']: raise Exception("Only sqlite is allowed for parameter 'category'")
        if category.lower() == 'sqlite':
            if type(file) == type(None): raise Exception("Parameter 'file' is mandatory")
            if type(table) == type(None): raise Exception("Parameter 'table' is mandatory")
            inapsw = apsw.Connection(file)
            inapsw.setrowtrace(dict_factory);
            tags = []
            fields = []
            for x in inapsw.cursor().execute("pragma table_info(" + table + ")"):
                if 'INTEGER' in x['type']: fields.append(x['name'])
                if 'NUMERIC' in x['type']: fields.append(x['name'])
                if 'VARCHAR' in x['type']: tags.append(x['name'])
            request = 'select '
            request += ','.join(tags) + ',' if len(tags) > 0 else ''
            request += ','.join(fields) if len(fields) > 0 else '0 ' + extracol
            request +=  ' from ' + table
            if len(fields) == 0: fields.append(extracol)
            buffer = []
            for row in inapsw.cursor().execute(request):
                dtags = dict()
                for x in tags: dtags[x] = row[x]
                dfields = dict()
                for x in fields: dfields[x] = row[x]
                d = dict(measurement = table, tags = dtags, fields = dfields, time=int(1000000000 * time.time()))
                if len(buffer) == batch:
                    s.write_points(buffer)
                    buffer = []
                buffer.append(d)
            if len(buffer) > 0: 
                s.write_points(buffer)
